#!/usr/bin/env python
# encoding: utf-8
'''
@license: (C) Copyright 2021, Hey.
@author: Hey
@email: sanyuan.hy@alibaba-inc.com
@tel: 137****6540
@datetime: 2023/4/21 20:57
@project: LucaProtApp
@file: inference_time
@desc: xxxx
'''
import os, sys, torch
import argparse
sys.path.append(".")
sys.path.append("../")
sys.path.append("../algorithms")
try:
    from predict_one_sample import load_args, predict_multi_label, \
        predict_multi_class, predict_binary_class, load_model
    from utils import fasta_reader, csv_writer, write_fasta, csv_reader, clean_seq
except ImportError:
    from algorithms.predict_one_sample import load_args, predict_multi_label, \
        predict_multi_class, predict_binary_class, load_model
    from algorithms.utils import fasta_reader, csv_writer, write_fasta, csv_reader, clean_seq

global_model_dir, global_config_dir = None, None

global_config, global_subword, global_seq_tokenizer, global_struct_tokenizer, global_model, global_label_id_2_name, global_label_name_2_id = None, None, None, None, None, None, None


def get_args():
    parser = argparse.ArgumentParser(description="Prediction")
    parser.add_argument("--fasta_file", default=None, type=str, help="fasta file path")
    parser.add_argument("--save_file", default=None, type=str, required=True, help="the result save file path")
    parser.add_argument("--seq_id", default=None, type=str, help="seq_id")
    parser.add_argument("--seq", default=None, type=str, help="one seq")
    parser.add_argument("--truncation_seq_length", default=4096, type=int, help="truncation seq length")
    parser.add_argument("--emb_dir", default=None, type=str, help="the structural embedding save dir. default: None")
    parser.add_argument("--dataset_name", default="rdrp_40_extend", type=str, help="the dataset name for model building.")
    parser.add_argument("--dataset_type", default="protein", type=str, help="the dataset type for model building.")
    parser.add_argument("--task_type", default="binary_class", type=str, choices=["multi_label", "multi_class", "binary_class"], help="the task type for model buliding.")
    parser.add_argument("--model_type", default="sefn", type=str, help="the model type for model building..")
    parser.add_argument("--time_str", default="20230201140320", type=str, help="the running time string(yyyymmddHimiss) of model building.")
    parser.add_argument("--step", default="100000", type=str, help="the training global step of model finalization.")
    parser.add_argument("--gpu_id", default=-1, type=int, help="the running gpu id (-1 for not using gpu)")
    parser.add_argument("--threshold",  default=0.5, type=float, help="sigmoid threshold for binary-class or multi-label classification, None for multi-class classification, defualt: 0.5.")
    parser.add_argument("--print_per_number", default=1000, type=int, help="print per number")
    cmd_args = parser.parse_args()
    return cmd_args


def main(args):
    global global_model_dir, global_config_dir, global_config, global_subword, global_seq_tokenizer, global_struct_tokenizer, global_model, global_label_id_2_name, global_label_name_2_id
    if args.print_per_number < 1:
        args.print_per_number = 1000
    model_dir = "../models/models/%s/%s/%s/%s/%s/%s" % (args.dataset_name, args.dataset_type, args.task_type,
                                                        args.model_type, args.time_str, args.step if args.step == "best" else "checkpoint-{}".format(args.step))
    config_dir = "../models/logs/%s/%s/%s/%s/%s" % (args.dataset_name, args.dataset_type, args.task_type,
                                                    args.model_type,  args.time_str)
    # Step1: loading the model configuration
    config = load_args(config_dir)
    print("-" * 25 + "config:" + "-" * 25)
    print(config)
    if config:
        args.dataset_name = config["dataset_name"]
        args.dataset_type = config["dataset_type"]
        args.task_type = config["task_type"]
        args.model_type = config["model_type"]
        args.has_seq_encoder = config["has_seq_encoder"]
        args.has_struct_encoder = config["has_struct_encoder"]
        args.has_embedding_encoder = config["has_embedding_encoder"]
        args.subword = config["subword"]
        args.codes_file = config["codes_file"]
        args.input_mode = config["input_mode"]
        args.label_filepath = config["label_filepath"]
        if not os.path.exists(args.label_filepath):
            args.label_filepath = os.path.join(config_dir, "label.txt")
        args.output_dir = config["output_dir"]
        args.config_path = config["config_path"]

        args.do_lower_case = config["do_lower_case"]
        args.sigmoid = config["sigmoid"]
        args.loss_type = config["loss_type"]
        args.output_mode = config["output_mode"]
        if args.gpu_id == -1 or args.gpu_id is None:
            args.device = torch.device("cpu")
        else:
            args.device = torch.device("cuda:%d" % args.gpu_id) if torch.cuda.is_available() else torch.device("cpu")
        args.seq_vocab_path = config["seq_vocab_path"]
        args.seq_pooling_type = config["seq_pooling_type"]
        args.seq_max_length = config["seq_max_length"]
        args.struct_vocab_path = config["struct_vocab_path"]
        args.struct_max_length = config["struct_max_length"]
        args.struct_pooling_type = config["struct_pooling_type"]
        args.trunc_type = config["trunc_type"]
        args.no_position_embeddings = config["no_position_embeddings"]
        args.no_token_type_embeddings = config["no_token_type_embeddings"]
        args.cmap_type = config["cmap_type"]
        args.cmap_type = float(config["cmap_thresh"])
        args.embedding_input_size = config["embedding_input_size"]
        args.embedding_pooling_type = config["embedding_pooling_type"]
        args.embedding_max_length = config["embedding_max_length"]
        args.embedding_type = config["embedding_type"]
        if args.task_type in ["multi-label", "multi_label", "binary-class", "binary_class"]:
            args.sigmoid = True

    print("-" * 25 + "args:" + "-" * 25)
    print(args.__dict__.items())
    print("-" * 25 + "model_dir list:" + "-" * 25)
    print(os.listdir(model_dir))

    # Step2: loading the tokenizer and model
    if global_model_dir != model_dir or global_model is None:
        global_config, global_subword, global_seq_tokenizer, global_struct_tokenizer, global_model, global_label_id_2_name, global_label_name_2_id = load_model(args, model_dir)
        print("LucaProt model loading success.")
    else:
        print("LucaProt model exists.")
    if args.task_type in ["multi-label", "multi_label"]:
        predict_func = predict_multi_label
    elif args.task_type in ["binary-class", "binary_class"]:
        predict_func = predict_binary_class
    elif args.task_type in ["multi-class", "multi_class"]:
        predict_func = predict_multi_class
    else:
        raise Exception("Not Support Task Type: %s" % args.task_type)

    # Step3: check output filepath
    if not os.path.exists(os.path.dirname(args.save_file)):
        print("the dirpath of save filepath: %s not exists." % os.path.dirname(args.save_file))
        os.makedirs(os.path.dirname(args.save_file))
    if os.path.exists(args.save_file):
        print("the save filepath: %s exists, continue prediction." % args.save_file)

    # Step4: prediction
    if os.path.isdir(args.save_file):
        print("the input --save_file=%s is a directory, please specify a file!")
        return
    tmp_file = args.save_file.replace(".fasta", "") \
        .replace(".fas", "") \
        .replace(".fa", "") \
        .replace(".csv", "") \
        .replace(".tsv", "") \
        .replace(".txt", "")
    csv_save_filepath = tmp_file + ".csv"
    positive_fasta_save_filepath = tmp_file + "_positives.fasta"
    total = 0
    if args.fasta_file is not None and os.path.exists(args.fasta_file):

        all_res = []
        positives = []
        done_seq_ids = set()
        if os.path.exists(csv_save_filepath):
            for row in csv_reader(csv_save_filepath, header=True, header_filter=True):
                seq_id = row[0]
                done_seq_ids.add(seq_id)
            print("exists done %d: " % len(done_seq_ids))
        for row in fasta_reader(args.fasta_file):
            seq_id = row[0]
            if seq_id in done_seq_ids:
                continue
            total += 1
            res = predict_func(args, global_label_id_2_name, global_seq_tokenizer, global_subword, global_struct_tokenizer, global_model, [row[0], clean_seq(row[0], row[1])])
            res = res[0]
            all_res.append(res[:4])
            if int(res[3]) == 1:
                positives.append([res[0], res[1]])
            if len(all_res) % args.print_per_number == 0:
                print("done: %d" % len(all_res))
                csv_writer(all_res, csv_save_filepath, header=["seq_id", "seq", "prob", "label"])
                if len(positives) > 0:
                    write_fasta(positive_fasta_save_filepath, positives)
                all_res = []
                positives = []
    elif args.seq is not None:
        total += 1
        positives = []
        if args.seq_id is None:
            args.seq_id = "unknown"
        row = [args.seq_id, args.seq]
        res = predict_func(args, global_label_id_2_name, global_seq_tokenizer, global_subword, global_struct_tokenizer, global_model, row)
        res = res[0]
        all_res = [res[:4]]
        if int(res[3]) == 1:
            positives.append([res[0], res[1]])
        print("result:")
        print(res)
    else:
        raise Exception("the input seq is null or fasta file is null")

    print("over: %d" % total)
    if len(all_res) > 0:
        csv_writer(all_res, csv_save_filepath, header=["seq_id", "seq", "prob", "label"])
    if len(positives) > 0:
        write_fasta(positive_fasta_save_filepath, positives)


if __name__ == "__main__":
    args = get_args()
    main(args)
    
